@@ -90,7 +90,11 @@ class LoginView(SuccessURLAllowedHostsMixin, FormView):
         return super(LoginView, self).dispatch(request, *args, **kwargs)
 
     def get_success_url(self):
-        """Ensure the user-originating redirection URL is safe."""
+        url = self.get_redirect_url()
+        return url or resolve_url(settings.LOGIN_REDIRECT_URL)
+
+    def get_redirect_url(self):
+        """Return the user-originating redirect URL if it's safe."""
         redirect_to = self.request.POST.get(
             self.redirect_field_name,
             self.request.GET.get(self.redirect_field_name, '')
@@ -100,9 +104,7 @@ class LoginView(SuccessURLAllowedHostsMixin, FormView):
             allowed_hosts=self.get_success_url_allowed_hosts(),
             require_https=self.request.is_secure(),
         )
-        if not url_is_safe:
-            return resolve_url(settings.LOGIN_REDIRECT_URL)
-        return redirect_to
+        return redirect_to if url_is_safe else ''
 
     def get_form_class(self):
         return self.authentication_form or self.form_class
@@ -121,7 +123,7 @@ class LoginView(SuccessURLAllowedHostsMixin, FormView):
         context = super(LoginView, self).get_context_data(**kwargs)
         current_site = get_current_site(self.request)
         context.update({
-            self.redirect_field_name: self.get_success_url(),
+            self.redirect_field_name: self.get_redirect_url(),
             'site': current_site,
             'site_name': current_site.name,
         })
@@ -131,12 +133,21 @@ class LoginView(SuccessURLAllowedHostsMixin, FormView):
 
 
 @deprecate_current_app
-def login(request, *args, **kwargs):
+def login(request, template_name='registration/login.html',
+          redirect_field_name=REDIRECT_FIELD_NAME,
+          authentication_form=AuthenticationForm,
+          extra_context=None, redirect_authenticated_user=False):
     warnings.warn(
         'The login() view is superseded by the class-based LoginView().',
         RemovedInDjango21Warning, stacklevel=2
     )
-    return LoginView.as_view(**kwargs)(request, *args, **kwargs)
+    return LoginView.as_view(
+        template_name=template_name,
+        redirect_field_name=redirect_field_name,
+        form_class=authentication_form,
+        extra_context=extra_context,
+        redirect_authenticated_user=redirect_authenticated_user,
+    )(request)
 
 
 class LogoutView(SuccessURLAllowedHostsMixin, TemplateView):
@@ -157,6 +168,10 @@ class LogoutView(SuccessURLAllowedHostsMixin, TemplateView):
             return HttpResponseRedirect(next_page)
         return super(LogoutView, self).dispatch(request, *args, **kwargs)
 
+    def post(self, request, *args, **kwargs):
+        """Logout may be done via POST."""
+        return self.get(request, *args, **kwargs)
+
     def get_next_page(self):
         if self.next_page is not None:
             next_page = resolve_url(self.next_page)
@@ -196,12 +211,20 @@ class LogoutView(SuccessURLAllowedHostsMixin, TemplateView):
 
 
 @deprecate_current_app
-def logout(request, *args, **kwargs):
+def logout(request, next_page=None,
+           template_name='registration/logged_out.html',
+           redirect_field_name=REDIRECT_FIELD_NAME,
+           extra_context=None):
     warnings.warn(
         'The logout() view is superseded by the class-based LogoutView().',
         RemovedInDjango21Warning, stacklevel=2
     )
-    return LogoutView.as_view(**kwargs)(request, *args, **kwargs)
+    return LogoutView.as_view(
+        next_page=next_page,
+        template_name=template_name,
+        redirect_field_name=redirect_field_name,
+        extra_context=extra_context,
+    )(request)
 
 
 _sentinel = object()
